@@ -3,25 +3,33 @@ import json
 import sys
 import logging
 import pandas as pd
-from pyodbc import connect
+import pyodbc
+import re
 
-# todo
-# driver_name = ''
-# driver_names = [x for x in pyodbc.drivers() if x.endswith(' for SQL Server')]
-# if driver_names:
-#     driver_name = driver_names[0]
-# if driver_name:
-#     conn_str = 'DRIVER={}; ...'.format(driver_name)
-#     # then continue with ...
-#     # pyodbc.connect(conn_str)
-#     # ... etc.
-# else:
-#     print('(No suitable driver found. Cannot connect.)')
+
+def __find_odbc_driver(name_pattern='.* SQL Server'):
+    """
+    Return name of ODBC driver matching name_pattern regex.
+
+    Keyword Arguments:
+        name_pattern {str} -- regex for driver name (default: {'.* SQL Server'})
+
+    Returns:
+        {str} -- name of ODBC driver
+    """
+    driver_names = [x for x in pyodbc.drivers() if re.search(
+        r"{}".format(name_pattern), x, flags=re.IGNORECASE)]
+
+    if driver_names:
+        return driver_names[0]
+    else:
+        return 'ODBC Driver 17 for SQL Server'
 
 
 def azure_get_credentials(**kwargs):
-    """ infers and/or updates defaults for Azure SQL and Azure storage account
-    defaults comes from credentials.json filepath set as environment 'AZURE_CREDENTIALS'
+    """
+    Infer and/or update defaults for Azure SQL and Azure storage account
+    defaults comes from credentials.json filepath set as environment variable 'AZURE_CREDENTIALS'
 
     Keyword Arguments:
         account_name {str} 'https://{account_name}.blob.core.windows.net/{container_name}/{blob_name}'
@@ -29,11 +37,10 @@ def azure_get_credentials(**kwargs):
         account_key {str} - account_key for storage authentication
 
         server {str} -- azure sql server url e.g "myazuresql.database.windows.net"
-        username {str} -- user id (default: admin )
-        password {str} -- password (default: admin_password)
+        username {str} -- user id (default: None )
+        password {str} -- password (default: None)
         database {str} -- database name (default: dev database)
 
-
     Returns:
     {dict} - credential dictionary
 
@@ -45,7 +52,7 @@ def azure_get_credentials(**kwargs):
             credentials = json.load(file)
     else:
         logging.warning(
-            'default credentials path does not exit, no defaults set for tableau server authentication')
+            'default credentials path does not exit, no defaults set for authentication')
         credentials = {}
 
     if 'account_name' in kwargs:
@@ -91,10 +98,11 @@ def azure_sql_con(**credentials):
     database = credentials.get('database')
     username = credentials.get('username')
     password = credentials.get('password')
+    driver_name = __find_odbc_driver(name_pattern='.* SQL Server')
 
     try:
-        connection = connect('DRIVER={};SERVER=tcp:{};database={};UID={};PWD={}'.format(
-            'ODBC Driver 17 for SQL Server', SERVER, database, username, password))
+        connection = pyodbc.connect('DRIVER={};SERVER=tcp:{};database={};UID={};PWD={}'.format(
+            driver_name, SERVER, database, username, password))
         logging.info('connected to {} as {}'.format(database, username))
         return connection
 
@@ -146,8 +154,9 @@ def azure_sql_engine(**credentials):
     database = credentials.get('database')
     username = credentials.get('username')
     password = credentials.get('password')
+    driver_name = __find_odbc_driver(name_pattern='.* SQL Server')
 
-    connection_string = quote_plus('DRIVER={ODBC Driver 17 for SQL Server}' +
+    connection_string = quote_plus(f'DRIVER={driver_name}' +
                                    f';SERVER=tcp:{server};DATABASE={database};UID={username};PWD={password}')
     engine = sqlalchemy.create_engine(
         f"mssql+pyodbc:///?odbc_connect={connection_string}")
@@ -182,13 +191,3 @@ def df_to_azure_sql(df, table_name, engine=None, if_exists='fail', index=False,
 
     df.to_sql(table_name, engine, index=index, if_exists=if_exists, **kwargs)
     return table_name
-
-
-def __parse_storage_url(blob_url):
-    account_name, container_name, blob_name = blob_url[8:].split('/')
-    account_name = account_name.split('.')[0]
-    return account_name, container_name, blob_name
-
-
-def __make_storage_url(account_name, container_name, blob_name):
-    return f'https://{account_name}.blob.core.windows.net/{container_name}/{blob_name}'
